/* $USAGI: rawrecv.c,v 1.1 2001/07/27 12:32:28 yoshfuji Exp $ */

/*
 * Copyright (C) 2001 USAGI/WIDE Project.
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the project nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE PROJECT AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE PROJECT OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/uio.h>
#include <netinet/in.h>
#include <netdb.h>
#include <getopt.h>

int main(int argc, char **argv){
	struct msghdr msghdr;
	struct iovec iov[2];
	char buf[128];
	struct sockaddr_storage sa;
	int s, cc;
	struct addrinfo hints, *res0;
	int gai;
	char *host, *serv;
	char abuf[NI_MAXHOST];	
	char *myname = argv[0];
	int off = -1;
	int usage = 0;

	while ((cc = getopt(argc, argv, "c:")) != EOF) {
		switch(cc){
		case 'c':
			off = atoi(optarg);
			if (off > (int)sizeof(buf))
				usage = 1;
			break;
		}
	}

	if (usage) {
		printf("%d,%d\n", off, usage);
		fprintf(stderr, "Usage: %s [-c csum_offset]\n", myname);
		exit(1);
	}

	memset(&hints, 0, sizeof(hints));
	hints.ai_family = PF_UNSPEC;
	hints.ai_socktype = SOCK_RAW;
	hints.ai_protocol = IPPROTO_RSVP;
	hints.ai_flags = AI_PASSIVE;
	gai = getaddrinfo("::", NULL, &hints, &res0);
	if (gai) {
		printf("getaddrinfo(): %s\n",
			gai_strerror(gai));
		exit(2);
	}

	s = socket(res0->ai_family, res0->ai_socktype, res0->ai_protocol);
	if (s < 0) {
		perror("socket");
		exit(2);
	}

	if (setsockopt(s, IPPROTO_IPV6, IPV6_CHECKSUM, &off, sizeof(off))) {
		perror("setsockopt(IPV6_CHECKSUM)");
		exit(1);
	}

	if (bind(s, res0->ai_addr, res0->ai_addrlen)) {
		freeaddrinfo(res0);
		perror("bind");
		exit(2);
	}
	freeaddrinfo(res0);

	while(1) {
		memset(&iov, 0, sizeof(iov));
		iov[0].iov_base = buf;
		iov[0].iov_len = sizeof(buf);

		memset(&msghdr, 0, sizeof(msghdr));
		msghdr.msg_name = (void *)res0->ai_addr;
		msghdr.msg_namelen = res0->ai_addrlen;
		msghdr.msg_iov = iov;
		msghdr.msg_iovlen = 1;
		msghdr.msg_control = NULL;
		msghdr.msg_controllen = 0;
		msghdr.msg_flags = 0;

		cc = recvmsg(s, &msghdr, 0);
		if (cc < 0) {
			perror("recvmsg");
			exit(2);
		}

		if (getnameinfo((struct sockaddr *)msghdr.msg_name, msghdr.msg_namelen,
				abuf, sizeof(abuf), NULL, 0,
				NI_NUMERICHOST))
			strcpy(abuf, "???");

		printf("received %d byte(s) from %s\n", cc, abuf);
	}
	exit(0);
}

